package simulator.workload.generator;

import java.io.FileNotFoundException;
import java.io.IOException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import simulator.ConfigurationOptions;
import simulator.workload.generator.impl.QcgJobGenerator;

public class WorkloadGenerator {
	
	private Log log = LogFactory.getLog(WorkloadGenerator.class);
	
	public static void main(String args[]){
		WorkloadGenerator generator = new WorkloadGenerator();
		generator.run(args);
	}
	
	public void run(String args[]){
		log.info("Start workload generator.");
		
		if(args == null || args.length == 0){
			log.error("Path to properties file is not provided. Can not configure workload generator.");
			return;
		}
		
		String propertiesFileName = args[0];
		ConfigurationOptions co = ConfigurationOptions.getConfiguration(propertiesFileName);
		if(co == null){
			log.error("Configuration options are not defined. Check file: "+propertiesFileName);
			return;
		}
		
		JobGenerator jobGenerator = new QcgJobGenerator();
		
		try {
			
			log.info("Generating workload.");
			jobGenerator.generateWorkload(co.workloadDescFileName, co.outputFolder, co.overwriteFiles, co.outputWorkloadFileName);
			
			log.info("Performing postprocessing.");
			jobGenerator.performPostprocessing(co.resdescFileName, co.outputFolder, co.outputWorkloadFileName);
			
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		
		log.info("End workload generator.");
	}
}
