package simulator.readers.validators;

import simulator.hostspec.ForecastFinishTimePluginName;
import simulator.hostspec.LocalAllocationPolicyPluginName;
import simulator.hostspec.Plugins;
import simulator.hostspec.QueueingSystem;
import simulator.hostspec.ResourcesConfiguration;
import simulator.hostspec.SingleResource;
import simulator.workload.exceptons.XMLLogicException;

/**
 * 
 * @author Marcin Krystek
 *
 */

public class ResourceConfigValidator extends XMLValidator{

	private ResourcesConfiguration resConfig;
	
	public ResourceConfigValidator(ResourcesConfiguration resConfig){
		this.resConfig = resConfig;
	}
	
	public boolean validate() throws XMLLogicException {
		SingleResource singleRes = resConfig.getSingleResource();
		QueueingSystem queueSys = resConfig.getQueueingSystem();
		
		boolean ret1 = false;
		boolean ret2 = false;
		
		ret1 = validateSingleResource(singleRes);
		ret2 = validateQueueingSystem(queueSys);
		
		return ret1 && ret2;
		
	}
	
	protected boolean validateQueueingSystem(QueueingSystem queueSys) throws XMLLogicException{
		if(queueSys == null)
			return true;
		
		int resCount = queueSys.getTotalResourceCount();
		int arResCount = queueSys.getARResourceCount();
		
		if(arResCount > resCount)
			throw new XMLLogicException("ARResourceCount should be less or equal to TotalResourceCount.");
		
		Plugins plugins = queueSys.getPlugins();
		checkPlugins(plugins, resCount);
		
		return true;
	}
	
	protected boolean validateSingleResource(SingleResource singleRes) throws XMLLogicException{
		if(singleRes == null)
			return true;
			
		int resCount = singleRes.getTotalResourceCount();
		int arResCount = singleRes.getARResourceCount();
		
		if(arResCount > resCount)
			throw new XMLLogicException("ARResourceCount should be less or equal to TotalResourceCount.");
		
		Plugins plugins = singleRes.getPlugins();
		checkPlugins(plugins, resCount);
		
		return true;
	}
	
	protected boolean checkPlugins(Plugins plugins, int resCount) throws XMLLogicException{
		
		ForecastFinishTimePluginName timePluginName[]  = plugins.getForecastFinishTimePluginName();
		LocalAllocationPolicyPluginName allocationPluginName[] = plugins.getLocalAllocationPolicyPluginName();
		if(timePluginName == null)
			throw new XMLLogicException("ForecastFinishTimePluginName element is compulsory.");
		if(allocationPluginName == null)
			throw new XMLLogicException("LocalAllocationPolicyPluginName element is compulsory.");
		
		int pluginResCount = 0;
		for(int i = 0; i < timePluginName.length; i++){
			pluginResCount += timePluginName[i].getResourcesCountWithThisPlugin();
		}
		if(pluginResCount != resCount)
			throw new XMLLogicException("Define ForecastFinishTimePluginName for all resources.");

		pluginResCount = 0;
		for(int i = 0; i < allocationPluginName.length; i++){
			pluginResCount += allocationPluginName[i].getResourcesCountWithThisPlugin();
		}
		if(pluginResCount != resCount)
			throw new XMLLogicException("Define LocalAllocationPolicyPluginName for all resources.");

		return true;
	}

}
